package de.unikiel.ipn.htmlform;

/**
 * A TextVisitor to remove line breaks.
 * They are replaced by spaces.
 */
class AntiLineBreakVisitor implements TextVisitor {
    private final String LINEBREAK = "\n";
    private final String LINEBREAK_REPLACEMENT = " ";
    // don't evaluate seperated words and linebreaks here
    private final int BORDER = 3;
    private final String END_SENTENCE = ".!?";
    private MessageHandler successor;

    /**
     * Creates an AntiLineBreakVisitor.
     * @param handler the MessageHandler for error messages
     * that may occur.
     */
    AntiLineBreakVisitor(MessageHandler handler) {
        successor = handler;
    }

    /**
     * Removes linebreaks from the content of the Entry.
     * Intended linebreaks at the end of a section are preserved.
     * They are identified by one of &quot;.!?&quot; at the end of
     * a line.
     * @param e the entry that shall be changed.
     */
    public void visit(Entry e) {
        int index;
        StringBuffer text = new StringBuffer(e.getContent());
        
        int preservedLinebreak = 0;
        while ((index = text.indexOf(LINEBREAK, preservedLinebreak)) != -1) {
            // evaluate seperated words, but not in the very beginning or end
            if (index + BORDER < text.length() 
                    && index > BORDER 
                    && text.charAt(index - 1) == '-' 
                    && Character.isLowerCase(text.charAt(index + 1))) {
                text.delete(index - 1, index + 1);
            }
            // evaluate real line breaks that must be preserved
            // that are linebreaks at the end of a sentence 
            else if (index + BORDER < text.length() 
                     && index > BORDER 
                     && ((END_SENTENCE.indexOf(text.charAt(index - 1)) != -1)
                         ||
                         (END_SENTENCE.indexOf(text.charAt(index - 2)) 
                          != -1))) {
                preservedLinebreak = index + 1;
                continue;
            }
            
            // evaluate normal line break
            else {
                text.replace(index, index + 1, LINEBREAK_REPLACEMENT);
            }
        }
        e.changeContent(text.toString());
    }

    /**
     * Forwards the Message to the successor.
     * See constructor.
     * @param msg the message that shall be passed to the successor. 
     */
    public void handleMessage(Message msg) {
        successor.handleMessage(msg);
    }
}

