package de.unikiel.ipn.htmlform;
import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
import javax.swing.text.*;

/**
 * An Entry.<br>
 * <i>Example:</i><br>
 * You have a html-file containing<br>
 * <pre>
 * &lt;html&gt;
 *  &lt;head&gt;
 *   &lt;title&gt;
 *    Protocol
 *   &lt;/title&gt;
 *  &lt;/head&gt;
 *  &lt;body&gt;
 *   &lt;!-- next log entry here --&gt;
 *  &lt;/body&gt;
 * &lt;/html&gt;
 * </pre><br>
 * Now you want to insert logs into the body right before the comment.
 * An <code>Entry</code> Object contains all information that is needed to 
 * place
 * the new text in the correct position, including the text itself.
 * The whole project file is mostly about how to create the Entry Objects.
 * See project file documentation.
 */
class Entry {
    private String prefix;
    private Document content;
    private String postfix;
    private File file;
    private String mark;
    private String append;
    private String label;
    private int leftAlignment;
    private int rightAlignment;
    private boolean isRegex;
    /**
     * keep track of all visitors
     */
    private List visitors = new ArrayList();
    
    /**
     * Creates a new Entry.
     * @param file name of the file to which the changed shall apply.
     *             <tt>new java.io.File(file).exists()</tt> must return true.
     * @throws FileNotFoundException if file is not the name of an existing file
     * @throws NullPointerException if file is <code>null</code>
     */
    Entry(String file) throws FileNotFoundException {
       this(null, null, null, file, null,  null, null, null, null);
    }
    
    /**
     * Creates a new Entry. 
     * @param prefix first part of the text that is inserted. It can not be
     *               changed. May be null.
     * @param content middle part of the text to be inserted. It can be changed
     *                in several ways ({@link #getDocument()}, 
     *                {@link #setContent(String)},
     *                {@link #changeContent(String)}). May be null.
     * @param postfix third part of the text to be inserted. It can not be
     *                changed. May be null.
     * @param file name of the file to which the changed shall apply.
     *             <tt>new java.io.File(file).exists()</tt> must return true.
     * @param mark a string in the file that marks the position where the
     *             prefix, content and postfix shall be inserted.
     * @param alignment the alignment for the content in the format
     *                  <tt>leftAlignment,rightAlignment</tt>. The text of the
     *                  content will have <tt>leftAlignment</tt> spaces in the
     *                  beginning of each line and will break at the first
     *                  space after <tt>rightAlignment</tt>. May be null for an
     *                  unaligned String.
     * @param append &quot;-1&quot; or null to insert the new text before the 
     *                              mark, 
     *               &quot;0&quot; to replace the mark with the new text,
     *               &quot;1&quot; to insert the new text after the mark.
     * @param isRegex &quot;0&quot; or null if mark is not a regular expression,
     *                &quot;1&quot; if mark is a regular expression.
     * @param label a label with no important meaning. Used to label the
     *              <tt>EntryPanel</tt>.
     * @throws FileNotFoundException if file is not the name of an existing file
     * @throws NullPointerException if file is null
     * @throws NumberFormatException if alignment string has a bad format
     */
    Entry(String prefix, String content, String postfix, String file,
          String mark, String alignment, String append, String isRegex,
          String label) throws FileNotFoundException {
        this.content = new PlainDocument();
        this.prefix = (prefix == null ? "" : prefix);
        setContent(content);
        if (getContent().equals("CURRENT_DATE")) {
            setContent(new SimpleDateFormat("dd.MM.yyyy").format(new Date()));
        }
        this.postfix = (postfix == null ? "" : postfix);
        this.mark = (mark == null ? "" : mark);
        this.file = new File(file); // may throw NullPointerException
        if (!this.file.exists()) {
            throw new FileNotFoundException("no such file: " + file.toString());
        }
        parseAlignment(alignment);
        this.append = 
            (append != null) && append.matches("(-1)|(0)|(1)") ? append : "-1";
        this.isRegex =
            (isRegex != null) && isRegex.equals("1") ? true : false;
        this.label = (label == null ? "" : label);
    }
    
    private void parseAlignment(String alignment) {
        if (alignment == null) {
            leftAlignment = 0;
            rightAlignment = 0;
        }
        else {
            String[] a = alignment.split("\\,");
            if (a.length != 2) {
                throw new NumberFormatException("precisely one comma"
                    + "required for alignment"
                );
            }
            leftAlignment = Integer.parseInt(a[0]);
            rightAlignment = Integer.parseInt(a[1]);
            if (leftAlignment >= rightAlignment) {
                throw new NumberFormatException
                    ("left alignment >= right alignment");
            }
            if (leftAlignment < 0 || rightAlignment < 0) {
                throw new NumberFormatException("negative alignment");
            }
        }
    }
    
    /**
     * Calls <code>v.visit(this)</code> on the argument.
     * If the TextVisitor has already visited this Entry,
     * then it is rejected <i>unless</i> the text has been
     * changed in the meantime by {@link #setContent(String)}.
     * @param v the TextVisitor that shall visit this Entry.
     * @see #setContent(String)
     * @see #changeContent(String)
     */
    synchronized void accept(TextVisitor v) {
        if (visitors.contains(v.getClass())) {
            // only accept each visitor once
            return;
        }
        visitors.add(v.getClass());
        v.visit(this);
    }
    
    /**
     * Replaces the content of this Entry.
     * All TextVisitors that have visited this Entry before
     * can visit it again after this method has been called.
     * @see #changeContent(String)
     * @see #accept(TextVisitor)
     */
    void setContent(String s) {
        visitors.clear();
        changeContent(s);
    }
    
    /**
     * Replaces the content of this Entry.
     * All TextVisitors that have visited this Entry before
     * are still blocked (can not visit it again).
     * @see #setContent(String)
     * @see #accept(TextVisitor)
     */
    final void changeContent(String s) {
        if (s == null) {
            s = "";
        }
        try {
            content.remove(0, content.getLength());
            content.insertString(0, s, null);
        } catch (BadLocationException e) {
            throw new IllegalStateException(
                "error when accessing a Document at " + e.offsetRequested()
            );
        }
    }
    
    String getPrefix() {
        return prefix;
    }
    
    String getPostfix() {
        return postfix;
    }
    
    /**
     * Returns the content of this Entry as a 
     * <tt>javax.swing.text.Document</tt>.
     * Changes to the returned <code>Document</code> 
     * will also change this Entry's
     * content and vice versa. 
     * @return a <tt>javax.swing.text.Document</tt> containing this Entry's
     * content
     * @see EntryPanel 
     */
    Document getDocument() {
        return content;
    }
    
    String getContent() {
        try {
            return content.getText(0, content.getLength());
        } catch (BadLocationException e) {
            throw new IllegalStateException(
                "error when accessing a Document at " + e.offsetRequested()
            );
        }
    }
    
    String getMark() {
        return mark;
    }
    
    File getFile() {
        return file;
    }
    
    int getLeftAlignment() {
        return leftAlignment;
    }

    int getRightAlignment() {
        return rightAlignment;
    }
    
    String getAppend() {
        return append;
    }
    
    boolean isRegex() {
        return isRegex;
    }

    String getLabel() {
        return label;
    }
}

