package de.unikiel.ipn.htmlform;
import javax.swing.*;
import java.awt.*;
import javax.swing.border.LineBorder;
import java.io.*;
import javax.help.*;
import java.net.URL;
import java.util.*;

/**
 * Main view.
 */
public class MainGUI extends JFrame {
    private static final MainGUI INSTANCE = new MainGUI();
    private static final int COMPONENT_HEIGHT = 60;
    
    // synchronize for print method
    private final Object PRINT_SYNCHRONIZER = new Object();
    public static final String VERSION="52";
    private JPanel textPanel = new JPanel();

    // A Runnable to be run in the event dispatching thread.
    // Used to display messages.
    private volatile Message lastMessage;
    private volatile Component lastMessageOwner;
    private Runnable printMessage = new Runnable() {
        public void run() {
            JOptionPane.showMessageDialog(lastMessageOwner,
                                          lastMessage.getMessage(),
                                          "message", 
                                          lastMessage.getType());
 
        }
    };

    // A simple scrollable JPanel.
    // It contains the EntryPanels.
    // See arrangeComponents(JComponent[])
    private static class ScrollPanel extends JPanel implements Scrollable {
        public Dimension getPreferredScrollableViewportSize() {
            return getPreferredSize();
        }

        public int getScrollableUnitIncrement(Rectangle visibleRect, 
                                              int o, int d) {
            return COMPONENT_HEIGHT;
        }

        public int getScrollableBlockIncrement(Rectangle visibleRect, 
                                               int o, int d) {
            return (int) visibleRect.getHeight();
        }
            
        public boolean getScrollableTracksViewportWidth() {
            return true;
        }

        public boolean getScrollableTracksViewportHeight() {
            return false;
        }
    }

    private MainGUI() {
        super("html-form version " + VERSION);
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            SwingUtilities.updateComponentTreeUI(this);
        } catch (UnsupportedLookAndFeelException e) {
            // doesn't matter, will continue with default l&f 
        } catch (ClassNotFoundException e2) {
            // doesn't matter, will continue with default l&f 
        } catch (IllegalAccessException e3) {
            // doesn't matter, will continue with default l&f 
        } catch (InstantiationException e4) {
            // doesn't matter, will continue with default l&f 
        }

        textPanel.setLayout(new BorderLayout());

        getContentPane().add(textPanel, BorderLayout.CENTER);
        setSize(400, 400);

        arrangeButtons();
        addWindowListener(InputController.getInstance());
    }
    
    /**
     * Returns always the same instance of this class.
     * This method is thread-safe. 
     * @return an instance of a MainGUI
     */
    static MainGUI getInstance() {
        return INSTANCE;
    }
    
    /**
     * Arranges the components to the pane.
     * Replaces the old ones that were added using this method.
     * @param components the JComponents that shall be displayed
     * @throws NullPointerException if components == null 
     */
    void arrangeComponents(JComponent[] components) {
        if (components == null) {
            throw new NullPointerException("argument components is null");
        }
        textPanel.removeAll();

        // the panel that contains the components
        ScrollPanel panel = new ScrollPanel();
        panel.setLayout(new GridBagLayout());

        // the constraints for that panel
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 0.5;
        c.gridx = 0;
        c.weighty = 0.5;
        c.ipady = COMPONENT_HEIGHT;

        for (c.gridy = 0; c.gridy < components.length; c.gridy++) {
            panel.add(components[c.gridy], c);
        }
        textPanel.add(new JScrollPane(
                          panel,
                          JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                          JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), 
                      BorderLayout.CENTER);
        textPanel.revalidate();
    }
    
    private void arrangeButtons() {
        JToolBar toolBar = new JToolBar();
        toolBar.setFloatable(false);

        JButton open = new JButton(new ImageIcon(getClass().
                                                  getResource("Open24.gif")));
        open.addActionListener(InputController.getInstance());
        open.setToolTipText("open a project file");
        open.setActionCommand("open");
        toolBar.add(open);
        
        JButton parse = new JButton(new ImageIcon(getClass().
                                                  getResource("Import24.gif")));
        parse.addActionListener(InputController.getInstance());
        parse.setToolTipText("import (parse) data from text");
        parse.setActionCommand("parse");
        toolBar.add(parse);
        
        JButton start = new JButton(new ImageIcon(getClass().
                                        getResource("RowDelete24.gif")));
        start.addActionListener(InputController.getInstance());
        start.setToolTipText("modify files");
        start.setActionCommand("start");
        toolBar.add(start);

        // init help button
        // Find the HelpSet file and create the HelpSet object:
        String helpHS = "de/unikiel/ipn/htmlform/help/HelpSet.hs";
        HelpSet hs = null;
        ClassLoader cl = MainGUI.class.getClassLoader();
        try {
            URL hsURL = HelpSet.findHelpSet(cl, helpHS);
            hs = new HelpSet(null, hsURL);
            // Create a HelpBroker object:
            HelpBroker hb = hs.createHelpBroker();
            JButton help = new JButton(new ImageIcon(getClass().
                                            getResource("Help24.gif")));
            help.addActionListener(new CSH.DisplayHelpFromSource(hb));
            help.setToolTipText("help");
            toolBar.add(help);
        } catch (HelpSetException hse) {
            print(null, new Message(
                    "Error initializing help features.\n"
                        + "Likely reason: Program files \n"
                        + "have been damaged.",
                    Message.ERROR_MESSAGE));
            hse.printStackTrace();
        }

        getContentPane().add(toolBar, BorderLayout.NORTH);
    }

    /**
     * Shows the message in a dialog.
     * The user has to confirm it before he can do anything else.
     * May be called from any thread. It does not return before
     * the user confirmed it.
     * It is thread-safe. 
     * @param owner the owner of the dialog.
     * @param msg the message that shall be displayed.
     */ 
    public void print(Component owner, Message msg) {
        synchronized (PRINT_SYNCHRONIZER) {
            System.out.println("entering lock");
            lastMessage = msg;
            lastMessageOwner = owner;
            if (SwingUtilities.isEventDispatchThread()) {
                System.out.println("edt prints " + msg);
                printMessage.run();
            }
            else {
                try {
                    SwingUtilities.invokeAndWait(printMessage);
                } catch (InterruptedException ie) {
                    // let's try again
                    print(owner, msg);
                } catch (java.lang.reflect.InvocationTargetException ite) {
                    // shouldn't be possible
                    System.err.println("printjob threw Exception:");
                    ite.getCause().printStackTrace();
                    System.exit(1);
                }
            }
        }
    }
}

