package de.unikiel.ipn.htmlform;
import java.io.File;
import tool.doc.*;

/**
 * Used to extract text from a pdf.
 * The extraction method is run(). It should be launched
 * in a seperate thread.<br> 
 */
class PDFParser implements Runnable, MessageHandler {
    private File file;
    private MessageHandler successor;

    /**
     * Create a new PDFParser.
     * @param file pdf-file to parse from
     * @param handler 
     * The run-method will pass Messages to this handler.
     * @throws NullPointerException if <code>file == null</code>
     * or <code>handler == null</code>
     * @see #run() 
     */
    PDFParser(File file, MessageHandler handler) {
        if (file == null || handler == null) {
            throw new NullPointerException("Arguments must not be null");
        }
        this.file = file;
        successor = handler;
    }

    /**
     * Extracts the text from the pdf.
     * When anything goes wrong the method will pass a 
     * {@link Message#ERROR_MESSAGE Message.ERROR_MESSAGE} to the
     * successing MessageHandler specified in the constructor.
     * Finally it will always pass a
     * {@link Message#RESULT_MESSAGE Message.RESULT_MESSAGE}
     * to the successing MessageHandler specified in the constructor.
     * That Message will contain the content of the parsed pdf
     * or an empty string if anything went wrong.
     * @see Message 
     */
    public void run() {
        String text = null;
        try {
            ExtractText et = new ExtractText();
            text = et.extract(file.toURI(), null);
            // kill mini spaces
            text = text.replaceAll("\uFB01 ", "fi");
            text = text.replaceAll("\uFB00 ", "ff");
            text = text.replaceAll("\uFB02 ", "fl");
            text = text.replaceAll("\uFB03 ", "ffi");
            text = text.replaceAll("\uFB04 ", "ffl");
            text = text.replaceAll("\uFB05 ", "ft");
            text = text.replaceAll("\uFB06 ", "st");
            text += 
                ("\nFILENAME:"
                + file.getName()
                + "\nFILESIZE:"
                + file.length());
        } catch (Exception e) {
            /*
             * the 3rd party library I am using actually throws Exception
             */
            handleMessage(new Message(
                "error in pdf extractor: \n" + e.getMessage(),
                Message.ERROR_MESSAGE
            ));
            text = "";
        } finally {
            handleMessage(new Message(
                text, Message.RESULT_MESSAGE));
        }
    }

    public void handleMessage(Message msg) {
        successor.handleMessage(msg);
    }
}

