package de.unikiel.ipn.htmlform;
import java.lang.reflect.InvocationTargetException;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/**
 * Displays a simple progress dialog.
 * The only option for the user is to cancel and exit the program.
 * All methods are <i>not</i> thread safe unless otherwise documented.
 */
public class ProgressDialog extends JDialog implements MessageHandler {
    private JProgressBar progressBar;
    private MessageHandler successor;
        
    /**
     * Creates an instance of a ProgressDialog. 
     * @param owner the owner of this Dialog. May be null.
     * @param handler the MessageHandler that shall be the successor
     * of this MessageHandler. This MessageHandler will only forward
     * Messages of type Message.RESULT_MESSAGE to the handler. It will
     * handle all other messages by itself (display them with this Dialog
     * as the owner). May be null, but then this MessageHandler can not
     * handle Messages of type Message.RESULT_MESSAGE.
     * @see javax.swing.JDialog#JDialog(Frame)
     * @see #handleMessage(Message) 
     */
    public ProgressDialog(Frame owner, MessageHandler handler) {
        super(owner, "stand by ...");
        arrangeComponents();
        successor = handler;
    }

    /**
     * Creates an instance of a ProgressDialog. 
     * @param owner the owner of this Dialog. May be null.
     * @param handler the MessageHandler that shall be the successor
     * of this MessageHandler. This MessageHandler will only forward
     * Messages of type Message.RESULT_MESSAGE to the handler. It will
     * handle all other messages by itself (display them with this Dialog
     * as the owner). May be null, but then this MessageHandler can not
     * handle Messages of type Message.RESULT_MESSAGE.
     * @see javax.swing.JDialog#JDialog(Dialog)
     * @see #handleMessage(Message) 
     */
    public ProgressDialog(Dialog owner, MessageHandler handler) {
        super(owner, "stand by ...");
        arrangeComponents();
        successor = handler;
    }
    
    private void arrangeComponents() {
        getContentPane().setLayout(new BorderLayout());
        
        JButton cancelButton = new JButton("Cancel and exit");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
                System.exit(1);
            }
        });

        progressBar = new JProgressBar();
        progressBar.setIndeterminate(true);
        progressBar.setStringPainted(true);
        progressBar.setString("");
        
        getContentPane().add(cancelButton, BorderLayout.PAGE_END);
        getContentPane().add(progressBar, BorderLayout.CENTER);

        setResizable(false);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        pack();
    }

    /**
     * Shows a ProgressDialog.
     * @param s description of the process to wait for (shown in progress bar) 
     * <br>may be null 
     */
    void showProgress(String s) {
        progressBar.setString(s);
        show();
    }

    /** 
     * Handles the message. 
     * This MessageHandler will forward
     * Messages of type Message.RESULT_MESSAGE to the handler. It will
     * handle all other messages by itself (display them in a seperate
     * Dialog with this Dialog as the owner).
     * @param msg the Message.
     * @throws IllegalStateException if the successor MessageHandler (see
     * constructor) is null and the Message is of type Message.RESULT_MESSAGE. 
     */
    public void handleMessage(Message msg) {
        switch (msg.getType()) {
            case Message.RESULT_MESSAGE :
                if (successor == null) {
                    throw new IllegalStateException(
                        "no successor specified to receive result message"
                    );
                }
                successor.handleMessage(msg);
                break;
            default :
                MainGUI.getInstance().print(this, msg);
        }
    }
}

