package de.unikiel.ipn.htmlform;
import java.io.*;
import java.text.*;
import java.util.*;
import java.util.regex.*;

/**
 * This Class provides a simple {@link Appender} implementation.
 * It changes files based only on the information
 * stored in the {@link Entry} objects.
 */

class SimpleAppender implements Appender, MessageHandler {
    private MessageHandler successor;

    /**
     * Creates a new instance.
     * @param handler the MessageHandler that shall handle occuring
     * error messages.
     * @throws NullPointerException if <code>handler == null</code>
     */
    SimpleAppender(MessageHandler handler) {
        if (handler == null) {
            throw new NullPointerException(
                "MessageHandler argument must not be null"
            );
        }
        successor = handler;
    }

    /**
     * Applies all changes that are stored in the entries.
     * Order from first to last entry is guaranteed.
     * @param entries an array containing the {@link Entry} objects 
     * @throws ArrayIndexOutOfBoundsException when entries.length == 0
     * @throws NullPointerException when entries == null
     * @see Entry 
     */
    public void append(Entry[] entries) {
        // special actions for first entry
        File file = entries[0].getFile();
        StringBuffer sb = readFile(file);
        if (sb == null) return;
        int index = 0;
        // jump to correct position
        try {
            index = jumpTo(index, sb, entries[0].getMark(), 
                           entries[0].isRegex(), entries[0].getAppend());
        } catch (PatternSyntaxException e) {
            handleMessage(new Message(
                "invalid regex:\n" + entries[0].getMark() + "\n"
                + "for entry " + 0,
                Message.ERROR_MESSAGE)
            );
            return;
        }
        if (index == -1) {
            handleMessage(new Message(
                "mark not found:\n" + entries[0].getMark() 
                + "\nin file " + entries[0].getFile().getName(),
                Message.ERROR_MESSAGE)
            );
            return;
        }
        
        for (int i = 0; i < entries.length; i++) {
            // different file for this entry?
            if (!file.equals(entries[i].getFile())) {
                writeFile(file, sb);
                file = entries[i].getFile();
                sb = readFile(file);
                if (sb == null) return;
                index = 0;
            }
            // jump to correct position
            try {
                index = 
                    jumpTo(index, sb, entries[i].getMark(), 
                           entries[i].isRegex(), entries[i].getAppend());
            } catch (PatternSyntaxException e) {
                handleMessage(new Message(
                    "invalid regex:\n" + entries[i].getMark() + "\n"
                    + "for entry " + i,
                    Message.ERROR_MESSAGE)
                );
                return;
            }
                
            if (index == -1) {
                handleMessage(new Message(
                    "mark not found:\n" + entries[i].getMark()  
                    + "\nin file " + entries[i].getFile().getName(),
                    Message.ERROR_MESSAGE)
                );
                return;
            }
            
            sb.insert(index, entries[i].getPrefix());
            index += entries[i].getPrefix().length();

            String content = breakLine(entries[i].getContent(),
                                       entries[i].getLeftAlignment(),
                                       entries[i].getRightAlignment());
            sb.insert(index, content);
            index += content.length();

            sb.insert(index, entries[i].getPostfix());
            index += entries[i].getPostfix().length();
        }
        writeFile(file, sb);
    }

    /**
     * Returns a StringBuffer that is like the Argument,
     * but with linebreaks. Further arguments are left alignment
     * (number of spaces in the start of each line) and the right
     * alignment (breaks at first space after rightAlign).
     */
    private String breakLine(String s, int leftAlign, int rightAlign) {
        if (leftAlign > rightAlign) {
            throw new IllegalArgumentException
                ("left alignment >= right alignment");
        }
        if (leftAlign < 0 || rightAlign < 0) {
            throw new IllegalArgumentException("negative alignment");
        }
        if (rightAlign == 0) {
            return s;
        }

        StringBuffer ret = new StringBuffer(s);
        StringBuffer left = new StringBuffer("");
        for (int j = 0; j < leftAlign; j++) {
            left.append(" ");
        }
        for (int i = 0; i < ret.length(); i++) {
            /* insert left align */
            ret.insert(i, left);
            i += left.length();
            
            /* break line at next space after right alignment */
            int position = left.length();
            for (; i < ret.length(); position++, i++) {
                if (ret.charAt(i) == ' ' && position > rightAlign) {
                    ret.replace(i, i, " \n");
                    i++; // " \n" longer than replaced char ' '
                    break;
                }
            }
        }
        return ret.toString();
    }
    
    /**
     * returns a StringBuffer that contains the
     * content of the file
     */
    private StringBuffer readFile(File f) {
        BufferedReader reader = null;
        StringBuffer ret = new StringBuffer("");
        String line;
        try {
            reader = new BufferedReader(
                new InputStreamReader(
                    new FileInputStream(f)
                )
            );
            while ((line = reader.readLine()) != null) {
                ret.append(line);
                ret.append("\n");
            }
        } catch (IOException e) {
            handleMessage(new Message("error reading file "
                + f.getName() + ":\n" + e,
                Message.ERROR_MESSAGE)
            );
            return null;
        } finally {
            try {
                if (reader != null) {
                    reader.close();
                }
            } catch (IOException e) {
                // I did my best to close the reader.
                // I can't do more than that.
                // Anyway, it's not that important if it failed or not.
            };
            return ret;
        }
    }
    
    /**
     * writes the content of the StringBuffer to the file
     */
    private void writeFile(File f, StringBuffer buffer) {
        Writer w = null;
        try {
            w = new BufferedWriter(new OutputStreamWriter(
                new FileOutputStream(f))
            );
            w.write(buffer.toString());
        } catch (IOException e) {
            handleMessage(new Message("error writing file "
                + f.getName() + ":\n" + e,
                Message.ERROR_MESSAGE)
            );
        } finally {
            try { w.close(); } catch (IOException e) {};
        }
    }
    
    /**
     * Returns index of first occurance of target in sb.
     * @return index of first occurance of target in sb.
     *         Index right after target if append.equals(&quot;-1&quot;),
     *         deletes target from buffer if append.equals(&quot;0&quot;),
     *         returns -1 if target is not found,
     *         returns index right before target otherwise.
     */
    private int jumpTo(int index, StringBuffer sb, String target, 
                       boolean isRegex, String append) {
        if (target.equals("")) {
            return index;
        }
        if (isRegex) {
            Pattern p = Pattern.compile(target);
            Matcher m = p.matcher(sb.toString());
            if (!m.find()) {
                return -1;
            }
            target = m.group();
        }
        index = sb.lastIndexOf(target);
        if (index == -1) {
            return -1;
        }
        if (append.equals("0")) {
            // replace option
            sb.delete(index, index + target.length());
        }
        else if (append.equals("1")) {
            // insert after option
            index += target.length();
        }
        return index;
    }

    /**
     * Forwards the Message to the successor.
     * See constructor.
     * @param msg the message that shall be passed to the successor. 
     */
    public void handleMessage(Message msg) {
        successor.handleMessage(msg);
    }
}

