package de.unikiel.ipn.htmlform;
import java.util.*;
import java.awt.Component;
import javax.swing.*;
import java.io.*;
import java.lang.reflect.*;
import javax.swing.filechooser.FileFilter;

/**
 * Main class file.
 */
public final class ZFDNModel extends Observable {
    private static final String alignPattern="\\d{1,2}\\,\\d{1,2}";
    
    private List entryPanels;
    private Entry[] e; // careful: are not initialized before they are used
    private Properties properties;

    private ProgressDialog progressDialog;

    private JFileChooser chooser;
    private FileFilter fileFilter = new FileFilter() {
        public boolean accept(File f) {
            if (f.isDirectory() || f.getName().matches(".*\\.prj")) {
                return true;
            }
            return false;
        }
        
        public String getDescription() {
            return "html-form project files (*.prj)";
        }
    };
        
    private static ZFDNModel INSTANCE = new ZFDNModel();
    
    private ZFDNModel() {
        chooser = new JFileChooser(System.getProperty("user.dir"));
        chooser.setFileFilter(fileFilter);
        chooser.setDialogTitle("choose a project file");
    }
    
    /**
     * Returns always the same instance of a ZFDNModel.
     * @return an instance of ZFDNModel. 
     */
    static ZFDNModel getInstance() {
        return INSTANCE;
    }
    
    /**
     * Parses content of the entries using the project-variable 
     * &quot;parser&quot;. See documentation for custom parser.
     * The source is a text the user can enter into a dialog
     * that pops up as a result of this method call. 
     * Usually that text would be copied into the text field
     * in the dialog.
     * @see AutoImportDialog 
     */
    void parse() {
        if (properties == null) {
            showErrorMessage("open a project file first");
            return;
        }
        String classname = properties.getProperty("parser");
        if (classname == null) {
            showErrorMessage("no parser found in project file (parser=...)");
            return;
        }
        Class parserClass;
        try {
            parserClass = Class.forName(classname);
        } catch (ClassNotFoundException e) {
            showErrorMessage("specified parser class not found");
            return;
        } catch (Throwable e) {
            // there are actually errors that can occur here
            // like NoClassDefFoundError that ARE recoverable!
            showErrorMessage(e.getClass().getName() + " in the specified "
                             + "parser class.\n"
                             + "Using: Class.forName(" + classname + ")\n"
                             + "Printing stacktrace to "
                             + "stderr");
            e.printStackTrace();
            return;
        }
        
        Object parser;
        try {
            parser = parserClass.newInstance();
        } catch (Exception e) {
            if (e instanceof RuntimeException) {
                throw (RuntimeException) e;
            }
            showErrorMessage("Exception instantiating parser class: " + e
                             + "\nprinting stack trace to stderr");
            e.printStackTrace();
            return;
        }

        /*
         * Show a dialog which asks the user for the text to
         * parse the content from.
         */
        String source = AutoImportDialog.getInstance().showDialog();
        if (source == null) {
            source = "";
        }
        
        /*
         * parse the text from the input string
         * using the parser class specified
         * in the project file (parser=...)
         */
        Iterator it = entryPanels.iterator();
        int i = 0;
        while (it.hasNext()) {
            EntryPanel e = (EntryPanel) it.next();
            Object parsedContent = null;
            try {
                parsedContent = parserClass.
                    getMethod("parseEntry" + (++i), new Class[]{String.class}).
                        invoke(parser, new String[]{source});
            } catch (IllegalAccessException e6) {
                showErrorMessage("IllegalAccessException for method "
                                 + "parseEntry" + i + " in "
                                 + parserClass.getName());
            } catch (IllegalArgumentException e2) {
                showErrorMessage("IllegalArgumentException for method "
                                 + "parseEntry" + i + " in "
                                 + parserClass.getName());
            } catch (InvocationTargetException e3) {
                showErrorMessage("The method "
                                 + "parseEntry" + i + " in \n"
                                 + parserClass.getName()
                                 + "threw a Throwable. Printing StackTrace\n"
                                 + "to stderr, but continue parsing.");
                e3.getTargetException().printStackTrace();
            } catch (NoSuchMethodException e4) {
                showErrorMessage("in the specified parser class "
                                 + parserClass.getName() + " there is no "
                                 + "method public String parseEntry"
                                 + i + "(String).\n"
                                 + "The program will continue, but you should "
                                 + "implement such a method. It may return "
                                 + "null, which would have no effect on the "
                                 + "existing content.");
            } catch (SecurityException e5) {
                showErrorMessage("SecurityException for method "
                                 + "parseEntry" + i + " in "
                                 + parserClass.getName() + "\n"
                                 + "printing StackTrace to stderr but\n"
                                 + "continuing program flow");
                e5.printStackTrace();
            }
            if (parsedContent != null) {
                e.getEntry().setContent(parsedContent.toString());
            }
        }
    }
    
    /**
     * Initiates the update of the files that shall be changed.
     * A progress dialog is shown during the update. 
     */
    void update() {
        if (entryPanels == null || entryPanels.size() == 0) {
            showErrorMessage("open a project file first");
            return;
        }
        e = new Entry[entryPanels.size()];
        for (int i = 0; i < e.length; i++) {
            e[i] = ((EntryPanel) entryPanels.get(i)).getEntry();
        }

        // show progress dialog
        progressDialog = new ProgressDialog(MainGUI.getInstance(), null);
        MainGUI.getInstance().setEnabled(false);
        progressDialog.showProgress("updating files");

        // start update in new thread
        new Thread() {
            public void run() {
                try {
                    new Updater(e, progressDialog).update();
                } finally {
                    // dispose progress dialog
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            progressDialog.dispose();
                            MainGUI.getInstance().setEnabled(true);
                            MainGUI.getInstance().show();
                            progressDialog = null;
                        }
                    });
                }
            }
        }.start();
    }

    private void showMessage(String message, int type) {
        MainGUI.getInstance().print(MainGUI.getInstance(), 
                                    new Message(message, type));
    }

    /**
     * Pops up an error message.
     * Owner is the active Frame or Dialog.
     * That could be the MainGUI, a ProgressDialog
     * or an AutoImportDialog. 
     * Is not thread-safe, but may be called by any thread. 
     * @param message the message that shall be printed 
     */
    private void showErrorMessage(String message) {
        showMessage(message, Message.ERROR_MESSAGE);
    }
    
    /**
     * Pops up an information message.
     * Owner is the active Frame or Dialog.
     * That could be the MainGUI, a ProgressDialog
     * or an AutoImportDialog. 
     * Is not thread-safe, but may be called by any thread. 
     * @param message the message that shall be printed
     */
    private void showInformationMessage(String message) {
        showMessage(message, Message.ERROR_MESSAGE);
    }
    
    /**
     * Reads the content of the project file.
     * @param file the project file
     * @throws FileNotFoundException if there is no such file 
     * @see #createEntryPanels() 
     */
    void readProjectFile(File file) {
        try {
            properties = new Properties();
            InputStream in = new FileInputStream(file);
            if (in == null) {
                throw new FileNotFoundException(
                    "no project file found with name " + file
                );
            }
            properties.load(in);
            in.close();
        } catch (IOException e) {
            showErrorMessage("error reading project file " + file
                             + ": " + e.getMessage()
            );
        }
    }
    
    /**
     * Returns the EntryPanels.
     * @return an array of JComponents which represent the view of the Entrys
     * @see EntryPanel
     */
    JComponent[] getComponents() {
        return (JComponent[]) entryPanels.toArray(new JComponent[0]);
    }
    
    /**
     * Creates the entries based on the information in the
     * project file. It also creates the EntryPanels.
     * @see Entry
     * @see EntryPanel
     * @see #readProjectFile(File) 
     */
    void createEntryPanels() {
        entryPanels = new ArrayList();
        String entry;
        String prefix;
        String content;
        String postfix;
        String file;
        String mark;
        String alignment;
        String append;
        String regex;
        String lastFile = properties.getProperty("entry1.file");
        if (lastFile == null) {
            showErrorMessage("error in project file: there must be an entry "
                             + "entry1.file"
            );
            return;
        }
        if (properties.getProperty("default.file") != null) {
            showErrorMessage("default value for file (default.file) "
                             + "not allowed"
            );
            return;
        }
        
        for (int i = 1; 
             (entry = properties.getProperty("entry" + i)) != null; i++) {
            prefix = properties.getProperty("entry" + i + ".prefix",
                                    properties.getProperty("default.prefix"));
            content = properties.getProperty("entry" + i + ".content",
                                    properties.getProperty("default.content"));
            postfix = properties.getProperty("entry" + i + ".postfix",
                                    properties.getProperty("default.postfix"));
            file = properties.getProperty("entry" + i + ".file");
            mark = properties.getProperty("entry" + i + ".mark",
                                    properties.getProperty("default.mark"));
            alignment = properties.getProperty("entry" + i + ".alignment",
                                   properties.getProperty("default.alignment"));
            append = properties.getProperty("entry" + i + ".append",
                                    properties.getProperty("default.append"));
            regex = properties.getProperty("entry" + i + ".regex",
                                    properties.getProperty("default.regex"));
            
            // it is ok to have no entryX.file=...
            if (file == null) {
                file = lastFile;
            }
            else {
                lastFile = file;
            }
            
            try {
                entryPanels.add(new EntryPanel(new Entry(
                    prefix, content, postfix, file, mark, alignment, append, 
                    regex, entry
                )));
            } catch (FileNotFoundException e) {
                showErrorMessage("wrong file property for entry " + i
                                 + ": file not found"
                );
                entryPanels.clear();
                return;
            } catch (NumberFormatException e2) {
                showErrorMessage("Bad format for alignment for entry " + i);
                entryPanels.clear();
                return;
            }
        }
    }
    
    /**
     * Shows a file chooser asking for a project file to open.
     * @return The chosen file, null if none has been chosen. 
     */
    File askForProjectFile() {
        SwingUtilities.updateComponentTreeUI(chooser);
        chooser.setFileFilter(fileFilter);
        int ret = chooser.showOpenDialog(MainGUI.getInstance());
        if (ret != JFileChooser.APPROVE_OPTION) {
            return null;
        }
        return chooser.getSelectedFile();
    }

    /**
     * Launches the program.
     * @param args args[0] may contain a path to a file.
     * Then it it processed (opened) before the GUI is shown.
     */ 
    public static void main(String[] args) {
        if (args.length > 0) {
            // project file specified as command line option
            File projectFile;
            projectFile = new File(args[0]);
            getInstance().readProjectFile(projectFile);
            getInstance().createEntryPanels();
            MainGUI.getInstance().arrangeComponents(
                getInstance().getComponents()
            );
        }
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                MainGUI.getInstance().show();
            }
        });
    }
}

